"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var last_1 = require("rxjs/operators/last");
var take_1 = require("rxjs/operators/take");
var App_1 = require("./App");
var createApp_1 = require("./createApp");
describe('frint  › App', function () {
    it('throws error when creating new instance without name', function () {
        chai_1.expect(function () {
            // tslint:disable-next-line:no-unused-expression
            new App_1.App({ name: undefined });
        }).to.throw(/Must provide `name` in options/);
    });
    it('gets name option value', function () {
        var app = new App_1.App({
            name: 'MyApp',
        });
        chai_1.expect(app.getName()).to.equal('MyApp');
    });
    it('exposes methods as class properties', function () {
        var methods = {
            foo: function () { return 'foo'; },
        };
        var app = new App_1.App({
            name: 'MyApp',
            methods: methods,
        });
        chai_1.expect(app.foo()).to.equal('foo');
    });
    it('does not overwrite app properties or methods with options methods', function () {
        var methods = {
            // tslint:disable-next-line:no-empty
            getName: function () { },
        };
        chai_1.expect(function () { return (new App_1.App({
            name: 'MyApp',
            methods: methods,
        })); }).to.throw(/Cannot overwrite app's `getName` property or method with options method/);
    });
    it('gets parent and root app', function () {
        var rootApp = new App_1.App({
            name: 'RootApp',
        });
        var childApp = new App_1.App({
            name: 'ChildApp',
            parentApp: rootApp,
        });
        var grandchildApp = new App_1.App({
            name: 'GrandchildApp',
            parentApp: childApp,
        });
        chai_1.expect(rootApp.getName()).to.equal('RootApp');
        chai_1.expect(childApp.getName()).to.equal('ChildApp');
        chai_1.expect(rootApp.getParentApps()).to.deep.equal([]);
        chai_1.expect(childApp.getParentApp()).to.deep.equal(rootApp);
        chai_1.expect(childApp.getRootApp()).to.deep.equal(rootApp);
        chai_1.expect(childApp
            .getParentApps()
            .map(function (x) { return x.getOption('name'); })).to.deep.equal([
            'RootApp'
        ]);
        chai_1.expect(grandchildApp.getParentApp()).to.deep.equal(childApp);
        chai_1.expect(grandchildApp.getRootApp()).to.deep.equal(rootApp);
        chai_1.expect(grandchildApp
            .getParentApps()
            .map(function (x) { return x.getOption('name'); })).to.deep.equal([
            'ChildApp',
            'RootApp',
        ]);
    });
    it('registers providers with direct values', function () {
        var app = new App_1.App({
            name: 'MyApp',
            providers: [
                { name: 'foo', useValue: 'fooValue' },
            ],
        });
        chai_1.expect(app.get('foo')).to.equal('fooValue');
    });
    it('registers providers with factory values', function () {
        var app = new App_1.App({
            name: 'MyApp',
            providers: [
                { name: 'foo', useFactory: function () { return 'fooValue'; } },
            ],
        });
        chai_1.expect(app.get('foo')).to.equal('fooValue');
    });
    it('registers providers with class values', function () {
        var Foo = /** @class */ (function () {
            function Foo() {
            }
            Foo.prototype.getValue = function () {
                return 'fooValue';
            };
            return Foo;
        }());
        var app = new App_1.App({
            name: 'MyApp',
            providers: [
                { name: 'foo', useClass: Foo },
            ],
        });
        chai_1.expect(app.get('foo').getValue()).to.equal('fooValue');
    });
    it('registers providers with dependencies', function () {
        var Baz = /** @class */ (function () {
            function Baz(_a) {
                var foo = _a.foo, bar = _a.bar;
                this.foo = foo;
                this.bar = bar;
            }
            Baz.prototype.getValue = function () {
                return "bazValue, " + this.foo + ", " + this.bar;
            };
            return Baz;
        }());
        var app = new App_1.App({
            name: 'MyApp',
            providers: [
                {
                    name: 'foo',
                    useValue: 'fooValue'
                },
                {
                    name: 'bar',
                    useFactory: function (_a) {
                        var foo = _a.foo;
                        return "barValue, " + foo;
                    },
                    deps: ['foo'],
                },
                {
                    name: 'baz',
                    useClass: Baz,
                    deps: ['foo', 'bar'],
                }
            ],
        });
        chai_1.expect(app.get('foo')).to.equal('fooValue');
        chai_1.expect(app.get('bar')).to.equal('barValue, fooValue');
        chai_1.expect(app.get('baz').getValue()).to.equal('bazValue, fooValue, barValue, fooValue');
    });
    it('returns services from Root that are cascaded', function () {
        var ServiceC = /** @class */ (function () {
            function ServiceC() {
            }
            ServiceC.prototype.getValue = function () {
                return 'serviceC';
            };
            return ServiceC;
        }());
        var Root = createApp_1.default({
            name: 'MyApp',
            providers: [
                {
                    name: 'serviceA',
                    useValue: 'serviceA',
                    scoped: true,
                    cascade: true,
                },
                {
                    name: 'serviceB',
                    useFactory: function () { return 'serviceB'; },
                    scoped: true,
                    cascade: true,
                },
                {
                    name: 'serviceC',
                    useClass: ServiceC,
                    cascade: true,
                },
                {
                    name: 'serviceCScoped',
                    useClass: ServiceC,
                    cascade: true,
                    scoped: true,
                },
                {
                    name: 'serviceD',
                    useValue: 'serviceD',
                    cascade: false,
                },
                {
                    name: 'serviceE',
                    useValue: 'serviceE',
                    cascade: true,
                    scoped: false,
                }
            ],
        });
        var App1 = createApp_1.default({ name: 'App1' });
        var root = new Root();
        root.registerApp(App1);
        var app = root.getAppInstance('App1');
        // expect(app.get('serviceA')).to.equal('serviceA');
        chai_1.expect(app.get('serviceB')).to.equal('serviceB');
        chai_1.expect(app.get('serviceC').getValue()).to.equal('serviceC');
        chai_1.expect(app.get('serviceD')).to.equal(null);
        chai_1.expect(app.get('serviceE')).to.equal('serviceE');
        chai_1.expect(app.get('serviceF')).to.equal(null);
        chai_1.expect(root.get('serviceF')).to.equal(null);
    });
    it('returns null when service is non-existent in both Child App and Root', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1);
        var serviceA = app
            .getAppInstance('App1')
            .get('serviceA');
        chai_1.expect(serviceA).to.equal(null);
    });
    it('gets container', function () {
        var app = new App_1.App({
            name: 'MyApp'
        });
        chai_1.expect(app.getContainer()).to.deep.equal(app.container);
    });
    it('gets providers definition list', function () {
        var app = new App_1.App({
            name: 'MyApp',
            providers: [
                { name: 'foo', useValue: 'fooValue' },
            ],
        });
        chai_1.expect(app.getProviders()).to.deep.equal([
            { name: 'foo', useValue: 'fooValue' },
        ]);
    });
    it('gets individual provider definition', function () {
        var app = new App_1.App({
            name: 'MyApp',
            providers: [
                { name: 'foo', useValue: 'fooValue' },
            ],
        });
        chai_1.expect(app.getProvider('foo')).to.deep.equal({ name: 'foo', useValue: 'fooValue' });
    });
    it('calls initialize during construction, as passed in options', function () {
        var called = false;
        var app = new App_1.App({
            name: 'MyApp',
            initialize: function () {
                called = true;
            }
        });
        chai_1.expect(app.getName()).to.equal('MyApp');
        chai_1.expect(called).to.equal(true);
    });
    it('calls beforeDestroy, as passed in options', function () {
        var called = false;
        var app = new App_1.App({
            name: 'MyApp',
            beforeDestroy: function () {
                called = true;
            }
        });
        app.beforeDestroy();
        chai_1.expect(app.getName()).to.equal('MyApp');
        chai_1.expect(called).to.equal(true);
    });
    it('registers apps', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1, {
            regions: ['sidebar'],
        });
        chai_1.expect(app.hasAppInstance('App1')).to.equal(true);
        chai_1.expect(app.getAppInstance('App1').getOption('name')).to.equal('App1');
    });
    it('registers apps, by overriding options', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1, {
            name: 'AppOne',
            regions: ['sidebar'],
        });
        chai_1.expect(app.hasAppInstance('AppOne')).to.equal(true);
        chai_1.expect(app.getAppInstance('AppOne').getOption('name')).to.equal('AppOne');
    });
    it('registers apps', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1, {
            regions: ['sidebar'],
        });
        chai_1.expect(app.hasAppInstance('App1')).to.equal(true);
        chai_1.expect(app.getAppInstance('App1').getOption('name')).to.equal('App1');
    });
    it('streams registered apps as a collection', function (done) {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1, {
            regions: ['sidebar'],
        });
        var apps$ = app.getApps$();
        apps$.subscribe(function (apps) {
            chai_1.expect(Array.isArray(apps)).to.equal(true);
            chai_1.expect(apps.length).to.equal(1);
            chai_1.expect(apps[0].name).to.equal('App1');
            done();
        });
    });
    it('streams registered apps as a collection, with region filtering', function (done) {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1, {
            regions: ['sidebar'],
        });
        var apps$ = app.getApps$('sidebar');
        apps$.subscribe(function (apps) {
            chai_1.expect(Array.isArray(apps)).to.equal(true);
            chai_1.expect(apps.length).to.equal(1);
            chai_1.expect(apps[0].name).to.equal('App1');
            done();
        });
    });
    it('gets app once available (that will be available in future)', function (done) {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var root = new Root();
        root.getAppOnceAvailable$('App1')
            .subscribe(function (app) {
            chai_1.expect(app.getName()).to.equal('App1');
            done();
        });
        root.registerApp(App1);
    });
    it('gets app once available (that is already available)', function (done) {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var root = new Root();
        root.registerApp(App1);
        root.getAppOnceAvailable$('App1')
            .subscribe(function (app) {
            chai_1.expect(app.getName()).to.equal('App1');
            done();
        });
    });
    it('gets app scoped by region', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var App2 = createApp_1.default({ name: 'App2' });
        var app = new Root();
        app.registerApp(App1, {
            regions: ['sidebar'],
        });
        app.registerApp(App2, {
            regions: ['footer'],
            multi: true,
        });
        chai_1.expect(app.getAppInstance('App1')).to.be.an('object');
        chai_1.expect(app.getAppInstance('App1', 'sidebar')).to.be.an('object');
        chai_1.expect(app.getAppInstance('App2')).to.equal(null);
        chai_1.expect(app.getAppInstance('App2', 'footer')).to.equal(null);
        app.instantiateApp('App2', 'footer', 'footer-123');
        chai_1.expect(app.getAppInstance('App2', 'footer', 'footer-123')).to.be.an('object');
    });
    it('throws error when registering same App twice', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1);
        chai_1.expect(function () {
            app.registerApp(App1);
        }).to.throw(/App 'App1' has been already registered before/);
    });
    it('checks for app instance availability', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        chai_1.expect(app.hasAppInstance('blah')).to.equal(false);
        app.registerApp(App1);
        chai_1.expect(app.hasAppInstance('App1')).to.equal(true);
    });
    it('throws error when trying to instantiate non-existent App', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var app = new Root();
        chai_1.expect(function () {
            app.instantiateApp('blah');
        }).to.throw(/No app found with name 'blah'/);
    });
    it('throws error when trying to destroy non-existent App', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var app = new Root();
        chai_1.expect(function () {
            app.destroyApp('blah');
        }).to.throw(/No app found with name 'blah'/);
    });
    it('throws error when trying to destroy existing App without instances', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1);
        app.destroyApp('App1');
        chai_1.expect(function () {
            app.destroyApp('App1');
        }).to.throw(/No instance with key 'default' found for app with name 'App1'./);
    });
    it('app instance is removed when destroyApp is called', function () {
        var Root = createApp_1.default({ name: 'MyApp' });
        var App1 = createApp_1.default({ name: 'App1' });
        var app = new Root();
        app.registerApp(App1);
        chai_1.expect(app.getAppInstance('App1')).to.not.be.null;
        chai_1.expect(app.hasAppInstance('App1')).to.be.true;
        app.destroyApp('App1');
        chai_1.expect(app.getAppInstance('App1')).to.be.null;
        chai_1.expect(app.hasAppInstance('App1')).to.be.false;
    });
    it('can accept additional lifecycle callbacks for Root Apps while instantiating, without overriding', function () {
        var counter = 0;
        var Root = createApp_1.default({
            name: 'RootApp',
            initialize: function () {
                counter += 1;
            },
        });
        // tslint:disable-next-line:no-unused-expression
        new Root({
            initialize: function () {
                counter += 1;
            },
        });
        chai_1.expect(counter).to.equal(2);
    });
    it('can accept additional lifecycle callbacks for Child Apps while registering, without overriding', function () {
        var counter = 0;
        var Root = createApp_1.default({
            name: 'RootApp',
        });
        var ChildApp = createApp_1.default({
            name: 'ChildApp',
            initialize: function () {
                counter += 1;
            },
        });
        var app = new Root();
        app.registerApp(ChildApp, {
            initialize: function () {
                counter += 1;
            },
        });
        chai_1.expect(counter).to.equal(2);
    });
    it('can update providers at lifecycle level', function () {
        var Root = createApp_1.default({
            name: 'RootApp',
            providers: [
                {
                    name: 'foo',
                    useValue: 'original foo',
                },
            ],
            initialize: function () {
                var foo = this.get('foo');
                this.container.register({
                    name: 'foo',
                    useValue: foo + " [updated]",
                });
            }
        });
        var app = new Root();
        chai_1.expect(app.get('foo')).to.equal('original foo [updated]');
    });
    it('can access and update providers from lifecycle callback defined while instantiating', function () {
        var Root = createApp_1.default({
            name: 'RootApp',
            providers: [
                {
                    name: 'foo',
                    useValue: 'original foo',
                },
            ],
            initialize: function () {
                var foo = this.get('foo');
                this.container.register({
                    name: 'foo',
                    useValue: foo + " [updatedFromCreateApp]",
                });
            }
        });
        var app = new Root({
            initialize: function () {
                var foo = this.get('foo');
                this.container.register({
                    name: 'foo',
                    useValue: foo + " [updatedFromInstantiation]",
                });
            }
        });
        chai_1.expect(app.get('foo')).to.equal('original foo [updatedFromCreateApp] [updatedFromInstantiation]');
    });
    it('can listen for child apps registration from a provider', function (done) {
        var Root = createApp_1.default({
            name: 'RootApp',
            providers: [
                {
                    name: '__EXEC__',
                    useFactory: function (_a) {
                        var app = _a.app;
                        app.getApps$().pipe(take_1.take(2), last_1.last()).subscribe(function (appsList) {
                            chai_1.expect(appsList.length).to.equal(1);
                            chai_1.expect(appsList[0].name).to.equal('ChildApp');
                            done();
                        });
                    },
                    deps: ['app'],
                },
            ],
        });
        var Child = createApp_1.default({
            name: 'ChildApp',
        });
        var testApp = new Root();
        testApp.registerApp(Child);
    });
});
//# sourceMappingURL=App.spec.js.map